/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __houdini_SurfaceAccessibleHoudini_h__
#define __houdini_SurfaceAccessibleHoudini_h__

namespace fe
{

/**************************************************************************//**
    @brief Houdini Surface Binding

	@ingroup houdini
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessibleHoudini : virtual public SurfaceAccessibleI
{
	public:
								SurfaceAccessibleHoudini(void):
									m_pGdp(NULL)							{}

virtual							~SurfaceAccessibleHoudini(void)				{}

								//* as SurfaceAccessibleI

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										String a_name)
								{
									sp<SurfaceAccessorHoudini> spAccessor=
											accessor();
									if(spAccessor->bind(a_element,a_name))
									{
										return spAccessor;
									}
									return sp<SurfaceAccessorI>(NULL);
								}

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										SurfaceAccessibleI::Attribute
										a_attribute)
								{
									sp<SurfaceAccessorHoudini> spAccessor=
											accessor();
									if(spAccessor->bind(a_element,a_attribute))
									{
										return spAccessor;
									}
									return sp<SurfaceAccessorI>(NULL);
								}

virtual	sp<SurfaceI>			surface(void);
virtual	sp<SurfaceI>			subSurface(U32 a_subIndex);

virtual	void					append(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										SpatialTransform& a_rTransform);

virtual	void					instance(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										std::vector<SpatialTransform>&
										a_rTransformArray);

								//* Houdini specific
		void					setGdp(const GU_Detail* a_pGdp)
								{
//									if(m_pGdp!=a_pGdp || m_pGdpChangeable)
									{
										m_pGdp=a_pGdp;
										m_pGdpChangeable=NULL;
										m_spSurfaceI=NULL;
									}

									if(m_spSurfaceI.isValid())
									{
										sp<SurfaceTrianglesHoudini> spSurface=
												m_spSurfaceI;
										if(spSurface.isValid())
										{
											spSurface->setGdp(a_pGdp);
										}
									}
								}
		void					setGdpChangeable(GU_Detail* a_pGdp)
								{
//									if(m_pGdp!=a_pGdp ||
//											m_pGdpChangeable!=a_pGdp)
									{
										m_pGdp=a_pGdp;
										m_pGdpChangeable=a_pGdp;
										m_spSurfaceI=NULL;
									}

									if(m_spSurfaceI.isValid())
									{
										sp<SurfaceTrianglesHoudini> spSurface=
												m_spSurfaceI;
										if(spSurface.isValid())
										{
											spSurface->setGdp(a_pGdp);
										}
									}
								}
const	GU_Detail*				gdp(void)			{ return m_pGdp; }
		GU_Detail*				gdpChangeable(void)	{ return m_pGdpChangeable; }

	private:
sp<SurfaceAccessorHoudini>		accessor(void)
								{
									if(!m_pGdp)
									{
										return sp<SurfaceAccessorI>();
									}
									sp<SurfaceAccessorHoudini> spAccessor(
											new SurfaceAccessorHoudini);
									if(m_pGdpChangeable)
									{
										spAccessor->setGdpChangeable(
												m_pGdpChangeable);
									}
									else
									if(m_pGdp)
									{
										spAccessor->setGdp(m_pGdp);
									}
									return spAccessor;
								}

const	GU_Detail*		m_pGdp;
		GU_Detail*		m_pGdpChangeable;

		sp<SurfaceI>	m_spSurfaceI;
};

} // namespace

#endif /* __houdini_SurfaceAccessibleHoudini_h__ */
