/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

using namespace fe;

void BloatOp::initialize(void)
{
	catalog<Real>("Expansion")=0.0;
	catalog<Real>("Expansion","low")= -1.0;
	catalog<Real>("Expansion","high")=1.0;

	catalog< sp<Component> >("Input Surface");
}

void BloatOp::handle(Record& a_rSignal)
{
	Real expansion=catalog<Real>("Expansion");

	//* access output surface (copy of input)
	sp<SurfaceAccessibleI> spSurfaceAccessibleI=surfaceOutput(a_rSignal);
	if(spSurfaceAccessibleI.isNull())
	{
		feLog("BloatOp::handle output surface not spSurfaceAccessibleI\n");
		catalog<String>("error")="inaccessible output";
		return;
	}

	sp<SurfaceAccessorI> spAccessorPoint=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	FEASSERT(spAccessorPoint.isValid());

	const U32 count=spAccessorPoint->count();

	sp<SurfaceAccessorI> spAccessorNormal=spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_normal);
	if(spAccessorNormal.isNull() || spAccessorNormal->count()!=count)
	{
		feLog("BloatOp::handle access to normals failed\n");
		catalog<String>("error")="normals required";
		return;
	}

	for(U32 index=0;index<count;index++)
	{
		const SpatialVector point=spAccessorPoint->spatialVector(index);
		const SpatialVector normal=spAccessorNormal->spatialVector(index);

		spAccessorPoint->set(index,point+normal*expansion);
	}
}
